/* Perfectspot - Wordperfect Graphics viewer
 * Copyright (C) 2007 Ariya Hidayat (ariya@kde.org)
 * Copyright (C) 2006 Ariya Hidayat (ariya@kde.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA
 *
 */

/* "This product is not manufactured, approved, or supported by
 * Corel Corporation or Corel Corporation Limited."
 */

#include <QtGui>

#include "canvas.h"
#include "renderer.h"

View::View(QWidget *parent): QFrame(parent), zoom(100), angle(0), renderAngle(0),
	graphicsRect(), renderer(0)
{
	updateContent();

	info = tr("Welcome to Perfectspot\nUse menu File, Open to load a graphics file");

#ifdef SMOOTH_ZOOM
	zoomTimeLine = new QTimeLine(200, this);
	zoomTimeLine->setCurveShape(QTimeLine::EaseInCurve);
	connect(zoomTimeLine, SIGNAL(frameChanged(int)), this, SLOT(doZoom(int)));
#endif

#ifdef SMOOTH_ROTATE
	rotateTimeLine = new QTimeLine(200, this);
	rotateTimeLine->setCurveShape(QTimeLine::EaseOutCurve);
	connect(rotateTimeLine, SIGNAL(frameChanged(int)), this, SLOT(doRotate(int)));
#endif

	adjustMargins();
	setMinimumSize(400, 300);

	setAutoFillBackground(true);
	setBackgroundRole(QPalette::Base);
	setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
	setLineWidth(1);

	setAttribute(Qt::WA_StaticContents, true);
	//setAttribute(Qt::WA_OpaquePaintEvent, true);
}

int View::zoomLevel() const
{
	return (int)zoom;
}

void View::setZoomLevel(int d)
{
	zoom = (double)d;
	doZoom((int)zoom);
}

int View::rotation() const
{
	return (int)angle;
}

void View::setRotation(int a)
{
	angle = (double)a;
	doRotate(a);
}

// does NOT change the zoom level, only update the graphics
void View::doZoom(int d)
{
	if (renderer)
	{
		graphicsRect.setSize(renderer->defaultSize()*d/100);
		updateContent();
		adjustMargins();
		update();
	}
}

// does NOT change the rotation angle, only update the graphics
void View::doRotate(int a)
{
	if (renderer)
	{
		renderAngle = a;
		update();
	}
}

void View::hideInfo()
{
	info = QString();
	update();
}

#define ZOOM_FACTOR 2

void View::zoomIn()
{
#ifdef SMOOTH_ZOOM
	if (zoomTimeLine->state() == QTimeLine::NotRunning)
	{
		zoomTimeLine->setFrameRange((int)zoom, (int)(zoom*ZOOM_FACTOR));
		zoomTimeLine->start();
		zoom = zoom*ZOOM_FACTOR;
	}
#else
	zoom = zoom*ZOOM_FACTOR;
	setZoomLevel((int)zoom);
#endif
	info = tr("Zoom to %1%").arg(zoom);
	QTimer::singleShot(800, this, SLOT(hideInfo()));
}

void View::zoomOut()
{
#ifdef SMOOTH_ZOOM
	if (zoomTimeLine->state() == QTimeLine::NotRunning)
	{
		zoomTimeLine->setFrameRange((int)zoom, (int)(zoom/ZOOM_FACTOR));
		zoomTimeLine->start();
		zoom = zoom/ZOOM_FACTOR;
	}
#else
	zoom = zoom/ZOOM_FACTOR;
	setZoomLevel((int)zoom);
#endif
	info = tr("Zoom to %1%").arg(zoom);
	QTimer::singleShot(800, this, SLOT(hideInfo()));
}

void View::rotateLeft()
{
#ifdef SMOOTH_ROTATE
	if (rotateTimeLine->state() == QTimeLine::NotRunning)
	{
		rotateTimeLine->setFrameRange((int)angle, (int)(angle-90));
		rotateTimeLine->start();
		angle = angle - 90;
		while (angle < 0)
			angle += 360;
	}
#else
	angle = angle - 90;
	while (angle < 0)
		angle += 360;
	setRotation((int)angle);
#endif
}

void View::rotateRight()
{
#ifdef SMOOTH_ROTATE
	if (rotateTimeLine->state() == QTimeLine::NotRunning)
	{
		rotateTimeLine->setFrameRange((int)angle, (int)(angle+90));
		rotateTimeLine->start();
		angle = angle + 90;
		while (angle < 0)
			angle += 360;
	}
#else
	angle = angle + 90;
	while (angle >= 360)
		angle -= 360;
	setRotation((int)angle);
#endif
}

void View::openFile(const QString &file)
{
	currentPath = file;
	delete renderer;

	QFileInfo fi(file);
	info = tr("Loading %1").arg(fi.fileName());
	QTimer::singleShot(1500, this, SLOT(hideInfo()));
	update();

	renderer = new Renderer(file, this);

	zoom = 100;
	angle = renderAngle = 0;
	graphicsRect.setSize(renderer->defaultSize());

	updateContent();
	adjustMargins();
	update();
}

// rerender the graphics according to graphicsRect.size()
// update the offscreen buffer, does not update the screen
void View::updateContent()
{
	if (renderer)
	{
		int w = graphicsRect.width();
		int h = graphicsRect.height();

		QImage img = QImage(QSize(w,h), QImage::Format_ARGB32_Premultiplied);
		QPainter p(&img);
		p.setViewport(0, 0, w, h);
		p.fillRect(0, 0, w, h, Qt::white);
		p.setRenderHint(QPainter::Antialiasing, true);
		renderer->render(&p);
		buffer = QPixmap::fromImage(img);
	}
	else
		buffer = QPixmap(0, 0);
}

void View::adjustMargins()
{
	update(graphicsRect);
	int w = qMax(0, (width() - graphicsRect.width())/2);
	int h = qMax(0, (height() - graphicsRect.height())/2);
	graphicsRect.moveTopLeft(QPoint(w, h));
	update(graphicsRect);
}

void View::resizeEvent(QResizeEvent *event)
{
	adjustMargins();
	QFrame::resizeEvent(event);
}

void View::paintEvent(QPaintEvent *event)
{
	QPainter painter(this);

	if (!graphicsRect.isNull())
	{
		painter.save();
		QPoint ctr = graphicsRect.center();
		QRect gr = graphicsRect.translated(-ctr.x(), -ctr.y());
		painter.setRenderHint(QPainter::Antialiasing, false);
		painter.translate(graphicsRect.center());
		painter.rotate(renderAngle);
		painter.setPen(QColor(224,224,224));
		painter.drawRect(gr.adjusted(-1,-1,1,1));
		painter.drawPixmap(gr, buffer);
		painter.restore();
	}

	if (!info.isEmpty())
	{
		int m = 5;
		int flags = Qt::AlignLeft | Qt::AlignVCenter;
		QRect r = painter.boundingRect(20, 20, 100, 20, flags, info);
		painter.setPen(Qt::black);
		painter.setBrush(QColor(255,255,220,192));
		painter.drawRect(r.adjusted(-m, -m, m, m));
		painter.drawText(r, flags, info);
	}

	QFrame::paintEvent(event);
}

void View::mousePressEvent(QMouseEvent *event)
{
	mousePressPos = event->pos();
	graphicsPos = graphicsRect.topLeft();
	event->accept();
}

void View::mouseMoveEvent(QMouseEvent *event)
{
	if (mousePressPos.isNull())
	{
		event->ignore();
		return;
	}

	graphicsRect.moveTopLeft(graphicsPos + event->pos() - mousePressPos);
	event->accept();
	update();
}

void View::mouseReleaseEvent(QMouseEvent *event)
{
	mousePressPos = QPoint();
	event->accept();
}

QSizeF View::graphicsSize() const
{
	int w = graphicsRect.width();
	int h = graphicsRect.height();
	double wf = w / 72.0;
	double hf = h / 72.0;
	return QSizeF(wf, hf);
}

bool View::exportGraphics(const QString &file)
{
	if (!renderer)
		return false;

	else if (file.toLower().endsWith(".svg"))
		return renderer->writeSVG(file);

	else if (file.toLower().endsWith(".odg"))
		return renderer->writeODG(file);

	else if (file.toLower().endsWith(".pdf"))
	{
		QPrinter printer(QPrinter::HighResolution);
		printer.setOutputFormat(QPrinter::PdfFormat);
		printer.setOutputFileName(file);
		//printer.setFullPage(true);
		printer.setPaperSize(renderer->defaultSize(), QPrinter::Point);

		QPainter painter;
		painter.begin(&printer);
		renderer->render(&painter);
		painter.end();

		return true;
	}

	// assume raster format, e.g. PNG, JPEG, ..
	else
		return buffer.save(file, 0, -1);
}

#include "canvas.moc"
