/* Perfectspot - Wordperfect Graphics viewer
 * Copyright (C) 2007 Ariya Hidayat (ariya@kde.org)
 * Copyright (C) 2006 Ariya Hidayat (ariya@kde.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA
 *
 */

/* "This product is not manufactured, approved, or supported by
 * Corel Corporation or Corel Corporation Limited."
 */

#include <QtGui>

#include "mainwindow.h"
#include "canvas.h"
#include "aboutbox.h"

class MainWindowPrivate
{
public:
	QString currentPath;
	View *view;
	QLabel *sizeLabel;
	QLabel *zoomLabel;

	QAction *openAction;
	QAction *exportAction;
	QAction *pdfAction;
	QAction *quitAction;
	QAction *zoomInAction;
	QAction *zoomOutAction;
	QAction *rotateLeftAction;
	QAction *rotateRightAction;
	QAction *aboutAction;
	QAction *websiteAction;
};

MainWindow::MainWindow()
	: QMainWindow()
{
	d = new MainWindowPrivate;
	createUI();

	d->view = new View(this);
	setCentralWidget(d->view);
	setWindowTitle(tr("Perfectspot"));
	setWindowIcon(QPixmap(":/perfectspot.png"));

	QTimer::singleShot(0, this, SLOT(initUI()));
	adjustSize();
}

// construct all UI objects
void MainWindow::createUI()
{
	QMenu *fileMenu = new QMenu(tr("&File"), this);
	d->openAction = fileMenu->addAction(tr("&Open..."));
	d->exportAction = fileMenu->addAction(tr("&Export..."));
	d->pdfAction = fileMenu->addAction(tr("Convert to &PDF..."));
	fileMenu->addSeparator();
	d->quitAction = fileMenu->addAction(tr("E&xit"));

	QMenu *viewMenu = new QMenu(tr("&View"), this);
	d->zoomInAction = viewMenu->addAction(tr("Zoom &in"));
	d->zoomOutAction = viewMenu->addAction(tr("Zoom &out"));
	viewMenu->addSeparator();
	d->rotateLeftAction = viewMenu->addAction(tr("Rotate &left"));
	d->rotateRightAction = viewMenu->addAction(tr("Rotate &right"));

	QMenu *helpMenu = new QMenu(tr("&Help"), this);
#if QT_VERSION >= 0x040200
	d->websiteAction = helpMenu->addAction(tr("Perfectspot &Web Site..."));
#endif
	d->aboutAction = helpMenu->addAction(tr("&About..."));

	menuBar()->addMenu(fileMenu);
	menuBar()->addMenu(viewMenu);
	menuBar()->addMenu(helpMenu);

	d->openAction->setIcon(QPixmap(":/open.png"));
	d->exportAction->setIcon(QPixmap(":/export.png"));
	d->pdfAction->setIcon(QPixmap(":/pdf.png"));
	d->zoomInAction->setIcon(QPixmap(":/zoomin.png"));
	d->zoomOutAction->setIcon(QPixmap(":/zoomout.png"));
	d->rotateLeftAction->setIcon(QPixmap(":/rotateleft.png"));
	d->rotateRightAction->setIcon(QPixmap(":/rotateright.png"));

	QWidget *spacer1 = new QWidget(this);
	QWidget *spacer2 = new QWidget(this);
	spacer1->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	spacer2->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	spacer1->setMinimumWidth(50);
	spacer2->setMinimumWidth(50);

	QToolBar *mainToolbar = addToolBar(tr("Toolbar"));
	mainToolbar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
	mainToolbar->addAction(d->openAction);
	mainToolbar->addAction(d->exportAction);
	mainToolbar->addWidget(spacer1);
	mainToolbar->addAction(d->zoomInAction);
	mainToolbar->addAction(d->zoomOutAction);
	mainToolbar->addWidget(spacer2);
	mainToolbar->addAction(d->rotateLeftAction);
	mainToolbar->addAction(d->rotateRightAction);
	mainToolbar->setBackgroundRole(QPalette::Window);
	mainToolbar->setAutoFillBackground(true);

	statusBar()->showMessage("Ready", 2000);
	statusBar()->setBackgroundRole(QPalette::Window);
	statusBar()->setAutoFillBackground(true);
}

// delayed initializations which won't effect visual experience
void MainWindow::initUI()
{
	connect(d->openAction, SIGNAL(triggered()), this, SLOT(openFile()));
	connect(d->exportAction, SIGNAL(triggered()), this, SLOT(exportFile()));
	connect(d->pdfAction, SIGNAL(triggered()), this, SLOT(exportPDF()));
	connect(d->quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
	connect(d->zoomInAction, SIGNAL(triggered()), this, SLOT(zoomIn()));
	connect(d->zoomOutAction, SIGNAL(triggered()), this, SLOT(zoomOut()));
	connect(d->rotateLeftAction, SIGNAL(triggered()), this, SLOT(rotateLeft()));
	connect(d->rotateRightAction, SIGNAL(triggered()), this, SLOT(rotateRight()));
	connect(d->aboutAction, SIGNAL(triggered()), this, SLOT(about()));
#if QT_VERSION >= 0x040200
	connect(d->websiteAction, SIGNAL(triggered()), this, SLOT(gotoWebSite()));
#endif

	d->openAction->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_O));
	d->exportAction->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_E));
	d->quitAction->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_Q));
	d->zoomInAction->setShortcut(QKeySequence(Qt::Key_Plus));
	d->zoomOutAction->setShortcut(QKeySequence(Qt::Key_Minus));
	d->rotateLeftAction->setShortcut(QKeySequence(Qt::Key_Left));
	d->rotateRightAction->setShortcut(QKeySequence(Qt::Key_Right));

	d->zoomLabel = new QLabel(statusBar());
	d->zoomLabel->setMargin(2);
	statusBar()->addPermanentWidget(d->zoomLabel);

	d->sizeLabel = new QLabel(statusBar());
	d->sizeLabel->setMargin(2);
	statusBar()->addPermanentWidget(d->sizeLabel);

	QTimer::singleShot(0, this, SLOT(updateUI()));
}

void MainWindow::updateUI()
{
	bool en = !d->currentPath.isEmpty();
	d->exportAction->setEnabled(en);
	d->pdfAction->setEnabled(en);
	d->zoomInAction->setEnabled(en);
	d->zoomOutAction->setEnabled(en);
	d->rotateLeftAction->setEnabled(en);
	d->rotateRightAction->setEnabled(en);

	if (d->view->zoomLevel() <= 25)
		d->zoomOutAction->setEnabled(false);
	if (d->view->zoomLevel() >= 800)
		d->zoomInAction->setEnabled(false);

	if (en)
	{
		double w = d->view->graphicsSize().width();
		double h = d->view->graphicsSize().height();
		d->sizeLabel->setText(QString(" Width: %1\"   Height: %2\" ").arg(w,0,'f',2).arg(h,0,'f',2));
		d->zoomLabel->setText(QString("Zoom: %1%").arg(d->view->zoomLevel()));
		d->sizeLabel->show();
		d->zoomLabel->show();
	}
	else
	{
		d->sizeLabel->hide();
		d->zoomLabel->hide();
	}
}

void MainWindow::zoomIn()
{
	d->view->zoomIn();
	updateUI();
}

void MainWindow::zoomOut()
{
	d->view->zoomOut();
	updateUI();
}

void MainWindow::rotateLeft()
{
	d->view->rotateLeft();
	updateUI();
}

void MainWindow::rotateRight()
{
	d->view->rotateRight();
	updateUI();
}

void MainWindow::gotoWebSite()
{
#if QT_VERSION >= 0x040200
	QDesktopServices::openUrl(QUrl(QString::fromLatin1("http://libwpg.sourceforge.net")));
#endif
}

void MainWindow::about()
{
	AboutBox *aboutBox = new AboutBox(this);
	aboutBox->exec();
	delete aboutBox;
}

void MainWindow::openFile(const QString &path)
{
	QString fileName;
	if (path.isNull())
		fileName = QFileDialog::getOpenFileName(this, tr("Open WPG File"),
		                                        d->currentPath, "*.wpg");
	else
		fileName = path;

	if (!fileName.isEmpty())
	{
		d->currentPath = QString::null;
		d->view->openFile(fileName);
		d->currentPath = fileName;
		QFileInfo fi(fileName);
		setWindowTitle(tr("%1 - Perfectspot").arg(fi.fileName()));

		QFileInfo info(fileName);
		statusBar()->showMessage(QString("%1 loaded").arg(info.fileName()), 2000);
		QTimer::singleShot(0, this, SLOT(updateUI()));
	}
}

void MainWindow::exportFile(const QString &path)
{
	QString filterBMP = tr("BMP - Windows Bitmap (*.bmp)");
	QString filterJPG = tr("JPEG - Joint Photographic Experts Group (*.jpg)") ;
	QString filterODG = tr("ODG - OpenDocument Graphics (*.odg)") ;
	QString filterPNG = tr("PNG - Portable Network Graphics (*.png)");
	QString filterPDF = tr("PDF - Portable Document Format (*.pdf)");
	QString filterSVG = tr("SVG - Scalable Vector Graphics (*.svg)");
	QString filterAll =  tr("All (*)");

	QString selectedFilter = filterSVG;
	QString filters = filterBMP + QString(";;");
	filters = filters + filterJPG + QString(";;");
	filters = filters + filterODG + QString(";;");
	filters = filters + filterPNG + QString(";;");
	filters = filters + filterPDF + QString(";;");
	filters = filters + filterSVG + QString(";;");
	filters = filters + filterAll;

	QString fileName;
	if (path.isNull())
		fileName = QFileDialog::getSaveFileName(this, tr("Export"), QString(), filters, &selectedFilter);
	else
		fileName = path;

	if (!fileName.isEmpty())
	{
		if (d->view->exportGraphics(fileName))
		{
			QFileInfo info(fileName);
			statusBar()->showMessage(tr("Exporting to %1").arg(info.fileName()), 2000);
		}
		else
			QMessageBox::critical(0, tr("Export Error"), tr("Can't export to '%1'").arg(fileName));
	}
}

void MainWindow::exportPDF(const QString &file)
{
	QString filterPDF = tr("PDF - Portable Document Format (*.pdf)");
	QString filterAll =  tr("All (*)");

	QString selectedFilter = filterPDF;
	QString filters = filterPDF + QString(";;");

	QString fileName;
	if (file.isNull())
		fileName = QFileDialog::getSaveFileName(this, tr("Convert to PDF"), QString(), filters, &selectedFilter);
	else
		fileName = file;

	if (!fileName.isEmpty())
	{
		if (d->view->exportGraphics(fileName))
		{
			QFileInfo info(fileName);
			statusBar()->showMessage(tr("Converting to %1").arg(info.fileName()), 2000);
		}
		else
			QMessageBox::critical(0, tr("Conversion Error"), tr("Can't convert to '%1'").arg(fileName));
	}
}

#include "mainwindow.moc"
