/* Perfectspot - Wordperfect Graphics viewer
 * Copyright (C) 2007 Ariya Hidayat (ariya@kde.org)
 * Copyright (C) 2006 Ariya Hidayat (ariya@kde.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA
 *
 */

/* "This product is not manufactured, approved, or supported by
 * Corel Corporation or Corel Corporation Limited."
 */

#include "renderer.h"

#include <iostream>

#include <librevenge/librevenge.h>
#include <librevenge-stream/librevenge-stream.h>
#include <libwpg/libwpg.h>

#include <QtGui>
#include <QFile>
#include <QtSvg>

#include "odg/FemtoZip.h"
#include "odg/OutputFileHelper.h"
#include "odg/StringDocumentHandler.h"

using namespace libwpg;

class OdgOutputFileHelper : public OutputFileHelper
{
public:
	OdgOutputFileHelper(const char *outFileName) :
		OutputFileHelper(outFileName) {}
	~OdgOutputFileHelper() {}

	bool convertDocument(librevenge::RVNGInputStream *input)
	{
		OdgGenerator collector;
		StringDocumentHandler stylesHandler, contentHandler, manifestHandler, settingsHandler;
		collector.addDocumentHandler(&contentHandler, ODF_CONTENT_XML);
		collector.addDocumentHandler(&manifestHandler, ODF_MANIFEST_XML);
		collector.addDocumentHandler(&settingsHandler, ODF_SETTINGS_XML);
		collector.addDocumentHandler(&stylesHandler, ODF_STYLES_XML);
		try
		{
			if (!libwpg::WPGraphics::parse(input, &collector))
				return false;
		}
		catch (...)
		{
			return false;
		}

		const char s_mimetypeStr[] = "application/vnd.oasis.opendocument.graphics";
		if (!writeChildFile("mimetype", s_mimetypeStr, (char)0) ||
		        !writeChildFile("META-INF/manifest.xml", manifestHandler.cstr()) ||
		        !writeChildFile("content.xml", contentHandler.cstr()) ||
		        !writeChildFile("settings.xml", settingsHandler.cstr()) ||
		        !writeChildFile("styles.xml", stylesHandler.cstr()))
			return false;

		librevenge::RVNGStringVector objects=collector.getObjectNames();
		for (unsigned i=0; i<objects.size(); ++i)
		{
			StringDocumentHandler objectHandler;
			if (collector.getObjectContent(objects[i], &objectHandler))
				writeChildFile(objects[i].cstr(), objectHandler.cstr());
		}
		return true;
	}
	bool isSupportedFormat(librevenge::RVNGInputStream *input)
	{
		try
		{
			return libwpg::WPGraphics::isSupported(input);
		}
		catch (...)
		{
			return false;
		}
	}
};

class RendererPrivate
{
public:
	QSize defaultSize;
	librevenge::RVNGInputStream *stream;
	QByteArray content;
};

Renderer::Renderer(const QString &fileName, QObject *parent): QObject(parent)
{
	d = new RendererPrivate;
	d->stream = 0;
	d->defaultSize = QSize(500, 500);

	QFile file(fileName);
	if (file.open(QFile::ReadOnly))
	{
		QByteArray buf = file.readAll();
		const unsigned char *data = (const unsigned char *)buf.data();
		int size = buf.size();
		d->stream = new ::librevenge::RVNGStringStream(data, size);
		if (d->stream && d->stream->isStructured())
		{
			if (d->stream->existsSubStream("PerfectOffice_MAIN"))
			{
				librevenge::RVNGInputStream *tempStream = d->stream->getSubStreamByName("PerfectOffice_MAIN");
				if (tempStream)
				{
					delete d->stream;
					d->stream = tempStream;
				}
			}
		}

		if (d->stream)
		{
			librevenge::RVNGStringVector vec;
			librevenge::RVNGSVGDrawingGenerator generator(vec, "");
			bool result = WPGraphics::parse(d->stream, &generator);
			if (!result || vec.empty() || vec[0].empty())
			{
				std::cerr << "ERROR: SVG Generation failed!" << std::endl;
				return;
			}

			librevenge::RVNGString tmpString("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
			tmpString.append("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"");
			tmpString.append(" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n");
			tmpString.append(vec[0].cstr());
			tmpString.append("\n");

			d->content = tmpString.cstr();
			QSvgRenderer tmpRenderer(d->content, parent);
			d->defaultSize = tmpRenderer.defaultSize();
		}
	}
}

Renderer::~Renderer()
{
	delete d->stream;
	delete d;
}

void Renderer::render(QPainter *target)
{
	if (d->stream)
	{
		QSvgRenderer tmpRenderer(d->content, this);
		tmpRenderer.render(target);
	}
}

bool Renderer::writeSVG(const QString &outputFile)
{
	QFile file(outputFile);
	if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
		return false;

	QTextStream out(&file);
	out << d->content.data();

	file.close();
	return true;
}

bool Renderer::writeODG(const QString &outputFile)
{
	const char *szOutFile = outputFile.toLocal8Bit().constData();

	OdgOutputFileHelper helper(szOutFile);
	if (!helper.isSupportedFormat(d->stream))
	{
		fprintf(stderr, "ERROR: We have no confidence that you are giving us a valid WordPerfect Graphics.\n");
		return false;
	}

	if (!helper.convertDocument(d->stream))
	{
		fprintf(stderr, "ERROR : Couldn't write convert the document\n");
		return false;
	}

	return true;
}

QSize Renderer::defaultSize() const
{
	return d->defaultSize;
}

#include "renderer.moc"
